@@ -27,8 +27,10 @@
 #include <QBrush>
 
 #include <KCursor>         //make()
+#include <KLocalizedString>
+
+#include "radialMap.h" // defines
 
-#include "builder.h"
 #include "part/Config.h"
 #include "part/fileTree.h"
 #define SINCOS_H_IMPLEMENTATION (1)
@@ -67,10 +69,33 @@ void RadialMap::Map::make(const Folder *tree, bool refresh)
     //slow operation so set the wait cursor
     QApplication::setOverrideCursor(Qt::WaitCursor);
 
+    //build a signature of visible components
     {
-        //build a signature of visible components
+        //**** REMOVE NEED FOR the +1 with MAX_RING_DEPTH uses
+        //**** add some angle bounds checking (possibly in Segment ctor? can I delete in a ctor?)
+        //**** this is a mess
+
         delete [] m_signature;
-        Builder builder(this, tree, refresh);
+        m_signature = new QList<Segment*>[m_visibleDepth + 1];
+
+        m_root = tree;
+
+        if (!refresh) {
+            m_minSize = (tree->size() * 3) / (PI * height() - MAP_2MARGIN);
+            findVisibleDepth(tree);
+        }
+
+        setRingBreadth();
+
+        // Calculate ring size limits
+        m_limits.resize(m_visibleDepth + 1);
+        const double size = m_root->size();
+        const double pi2B  = M_PI * 4 * m_ringBreadth;
+        for (uint depth = 0; depth <= m_visibleDepth; ++depth) {
+            m_limits[depth] = uint(size / double(pi2B * (depth + 1))); //min is angle that gives 3px outer diameter for that depth
+        }
+
+        build(tree);
     }
 
     //colour the segments
@@ -89,12 +114,92 @@ void RadialMap::Map::setRingBreadth()
     //FIXME called too many times on creation
 
     m_ringBreadth = (height() - MAP_2MARGIN) / (2 * m_visibleDepth + 4);
+    m_ringBreadth = qBound(MIN_RING_BREADTH, m_ringBreadth, MAX_RING_BREADTH);
+}
 
-    if (m_ringBreadth < MIN_RING_BREADTH)
-        m_ringBreadth = MIN_RING_BREADTH;
+void RadialMap::Map::findVisibleDepth(const Folder *dir, uint currentDepth)
+{
 
-    else if (m_ringBreadth > MAX_RING_BREADTH)
-        m_ringBreadth = MAX_RING_BREADTH;
+    //**** because I don't use the same minimumSize criteria as in the visual function
+    //     this can lead to incorrect visual representation
+    //**** BUT, you can't set those limits until you know m_depth!
+
+    //**** also this function doesn't check to see if anything is actually visible
+    //     it just assumes that when it reaches a new level everything in it is visible
+    //     automatically. This isn't right especially as there might be no files in the
+    //     dir provided to this function!
+
+    static uint stopDepth = 0;
+
+    if (dir == m_root) {
+        stopDepth = m_visibleDepth;
+        m_visibleDepth = 0;
+    }
+
+    if (m_visibleDepth < currentDepth) m_visibleDepth = currentDepth;
+    if (m_visibleDepth >= stopDepth) return;
+
+    for (File *file : dir->files) {
+        if (file->isFolder() && file->size() > m_minSize) {
+            findVisibleDepth((Folder *)file, currentDepth + 1); //if no files greater than min size the depth is still recorded
+        }
+    }
+}
+
+//**** segments currently overlap at edges (i.e. end of first is start of next)
+bool RadialMap::Map::build(const Folder * const dir, const uint depth, uint a_start, const uint a_end)
+{
+    //first iteration: dir == m_root
+
+    if (dir->children() == 0) //we do fileCount rather than size to avoid chance of divide by zero later
+        return false;
+
+    FileSize hiddenSize = 0;
+    uint hiddenFileCount = 0;
+
+    for (File *file : dir->files) {
+        if (file->size() < m_limits[depth] * 6) { // limit is half a degree? we want at least 3 degrees
+            hiddenSize += file->size();
+            if (file->isFolder()) { //**** considered virtual, but dir wouldn't count itself!
+                hiddenFileCount += static_cast<const Folder*>(file)->children(); //need to add one to count the dir as well
+            }
+            ++hiddenFileCount;
+            continue;
+        }
+
+        unsigned int a_len = (unsigned int)(5760 * ((double)file->size() / (double)m_root->size()));
+
+        Segment *s = new Segment(file, a_start, a_len);
+        m_signature[depth].append(s);
+
+        if (file->isFolder()) {
+            if (depth != m_visibleDepth) {
+                //recurse
+                s->m_hasHiddenChildren = build((Folder*)file, depth + 1, a_start, a_start + a_len);
+            } else {
+                s->m_hasHiddenChildren = true;
+            }
+        }
+
+        a_start += a_len; //**** should we add 1?
+    }
+
+    if (hiddenFileCount == dir->children() && !Config::showSmallFiles) {
+        return true;
+    }
+
+    if ((depth == 0 || Config::showSmallFiles) && hiddenSize >= m_limits[depth]) {
+        //append a segment for unrepresented space - a "fake" segment
+        const QString s = i18np("1 file, with an average size of %2",
+                "%1 files, with an average size of %2",
+                hiddenFileCount,
+                KFormat().formatByteSize(hiddenSize/hiddenFileCount));
+
+
+        (m_signature + depth)->append(new Segment(new File(s.toUtf8(), hiddenSize), a_start, a_end - a_start, true));
+    }
+
+    return false;
 }
 
 bool RadialMap::Map::resize(const QRect &rect)
@@ -160,10 +265,9 @@ void RadialMap::Map::colorise()
 
     for (uint i = 0; i <= m_visibleDepth; ++i, darkness += 0.04)
     {
-        for (Iterator<Segment> it = m_signature[i].iterator(); it != m_signature[i].end(); ++it)
-        {
+        for (Segment *segment : m_signature[i]) {
             if (m_summary){ // Summary view has its own colors, special cased.
-                if ((*it)->file()->name() == QLatin1String("Used")) {
+                if (segment->file()->name() == QLatin1String("Used")) {
                     cb = QApplication::palette().highlight().color();
                     cb.getHsv(&h, &s1, &v1);
 
@@ -182,7 +286,7 @@ void RadialMap::Map::colorise()
                     cb = Qt::white;
                 }
 
-                (*it)->setPalette(cp, cb);
+                segment->setPalette(cp, cb);
             }
             else
             {
@@ -193,7 +297,7 @@ void RadialMap::Map::colorise()
                     //gradient will work by figuring out rgb delta values for 360 degrees
                     //then each component is angle*delta
 
-                    int a = (*it)->start();
+                    int a = segment->start();
 
                     if (a > 2880) a = 2880 - (a - 2880);
 
@@ -210,11 +314,11 @@ void RadialMap::Map::colorise()
                 case Filelight::HighContrast:
                     cp.setHsv(0, 0, 0); //values of h, s and v are irrelevant
                     cb.setHsv(180, 0, int(255.0 * contrast));
-                    (*it)->setPalette(cp, cb);
+                    segment->setPalette(cp, cb);
                     continue;
 
                 default:
-                    h  = int((*it)->start() / 16);
+                    h  = int(segment->start() / 16);
                     s1 = 160;
                     v1 = (int)(255.0 / darkness); //****doing this more often than once seems daft!
                 }
@@ -224,12 +328,12 @@ void RadialMap::Map::colorise()
 
                 if (s1 < 80) s1 = 80; //can fall too low and makes contrast between the files hard to discern
 
-                if ((*it)->isFake()) //multi-file
+                if (segment->isFake()) //multi-file
                 {
                     cb.setHsv(h, s2, (v2 < 90) ? 90 : v2); //too dark if < 100
                     cp.setHsv(h, 17, v1);
                 }
-                else if (!(*it)->file()->isFolder()) //file
+                else if (!segment->file()->isFolder()) //file
                 {
                     cb.setHsv(h, 17, v1);
                     cp.setHsv(h, 17, v2);
@@ -240,7 +344,7 @@ void RadialMap::Map::colorise()
                     cp.setHsv(h, s2, v2); //v was 225 - delta
                 }
 
-                (*it)->setPalette(cp, cb);
+                segment->setPalette(cp, cb);
 
                 //TODO:
                 //**** may be better to store KDE colours as H and S and vary V as others
@@ -301,19 +405,18 @@ void RadialMap::Map::paint(bool antialias)
         //clever geometric trick to find largest angle that will give biggest arrow head
         uint a_max = int(acos((double)width / double((width + 5))) * (180*16 / M_PI));
 
-        for (ConstIterator<Segment> it = m_signature[x].constIterator(); it != m_signature[x].end(); ++it)
-        {
+        for (Segment *segment : m_signature[x]) {
             //draw the pie segments, most of this code is concerned with drawing the little
             //arrows on the ends of segments when they have hidden files
 
-            paint.setPen((*it)->pen());
+            paint.setPen(segment->pen());
 
-            if ((*it)->hasHiddenChildren())
+            if (segment->hasHiddenChildren())
             {
                 //draw arrow head to indicate undisplayed files/directories
                 QPolygon pts(3);
                 QPoint pos, cpos = rect.center();
-                uint a[3] = { (*it)->start(), (*it)->length(), 0 };
+                uint a[3] = { segment->start(), segment->length(), 0 };
 
                 a[2] = a[0] + (a[1] / 2); //assign to halfway between
                 if (a[1] > a_max)
@@ -336,14 +439,14 @@ void RadialMap::Map::paint(bool antialias)
                     pts.setPoint(i, pos);
                 }
 
-                paint.setBrush((*it)->pen());
+                paint.setBrush(segment->pen());
                 paint.drawPolygon(pts);
             }
 
-            paint.setBrush((*it)->brush());
-            paint.drawPie(rect, (*it)->start(), (*it)->length());
+            paint.setBrush(segment->brush());
+            paint.drawPie(rect, segment->start(), segment->length());
 
-            if ((*it)->hasHiddenChildren())
+            if (segment->hasHiddenChildren())
             {
                 //**** code is bloated!
                 paint.save();
@@ -354,7 +457,7 @@ void RadialMap::Map::paint(bool antialias)
                 QRect rect2 = rect;
                 width /= 2;
                 rect2.adjust(width, width, -width, -width);
-                paint.drawArc(rect2, (*it)->start(), (*it)->length());
+                paint.drawArc(rect2, segment->start(), segment->length());
                 paint.restore();
             }
         }
